package jsyntaxpane.lexers;
import jsyntaxpane.Token;
import jsyntaxpane.TokenType;


%% 
%public
%class AM1Lexer
%extends DefaultJFlexLexer
%final
%unicode
%char
%type Token

%{
    /**
     * Create an empty lexer, yyrset will be called later to reset and assign
     * the reader
     */

    public AM1Lexer() {
        super();
    }

    private Token token(TokenType type) {
        return new Token(type, yychar, yylength());
    }

    private Token token(TokenType type, int pairValue) {
        return new Token(type, yychar, yylength(), (byte)pairValue);
    }

    private static final byte PARAN     = 1;
    private static final byte BRACKET   = 2;
    private static final byte CURLY     = 3;
%}


/* main character classes */
LineTerminator = \r|\n|\r\n
InputCharacter = [^\r\n]

/* whitespace */
Whitespace = {LineTerminator} | [ \t\f]+

/* comments */
Comment = {TraditionalComment} | {EndOfLineComment} 
TraditionalComment = "/*" [^*] ~"*/" | "/*" "*"+ "/"
EndOfLineComment = "//" {InputCharacter}* {LineTerminator}?

/* identifiers */
Identifier = [:jletter:][:jletterdigit:]*

/* integer literals */
Integer = 0 | [1-9][0-9]*

%%
<YYINITIAL> {
  {Whitespace}+       			 { /* ignore */ }
  
  /* keywords */
  /* arithmetic logic */
  "ADD"                          |
  "MUL"                          |
  "SUB"                          |
  "DIV"                          |
  "MOD"                          |
  "EQ"                           |
  "NE"                           |
  "LT"                           |
  "GT"                           |
  "LE"                           |
  "GE"                        	 |
  
  /* transport */
  "LOAD"                         |
  "LOADA"                        |
  "LOADI"                        |
  "STORE"                        |
  "STOREI"                       |
  "LIT"                          |
  
  /* jump */
  "JMP"                          |
  "JMC"                          |
  
  /* read write */
  "WRITE"                        |
  "WRITEI"                       |
  "READ"                         |
  "READI"						 |
  
  /* procedure calls */
  "PUSH"                         |
  "CALL"           				 |
  "INIT"                         |
  "RET"                          { return token(TokenType.KEYWORD); }

  /* operators */
  "("                            { return token(TokenType.OPERATOR,  PARAN); }
  ")"                            { return token(TokenType.OPERATOR, -PARAN); }
  ";"                            | 
  ","                            | 
  "-"                            { return token(TokenType.OPERATOR); } 
  
  /* lokal global*/
  "lokal"						 |
  "global"						 { return token(TokenType.STRING); } 

  /* numeric literals */
  {Integer}               		 { return token(TokenType.NUMBER); }

  /* comments */
  {Comment}                      { return token(TokenType.COMMENT); }

  /* identifiers */ 
  {Identifier}                   { return token(TokenType.IDENTIFIER); }
}

/* error fallback */
.|\n                             {  }
<<EOF>>                          { return null; }
